
from twisted.internet.defer import succeed
from yaml import dump

from ensemble.providers.dummy import MachineProvider
from ensemble.control import main

from .common import ControlToolTest


class ControlBootstrapTest(ControlToolTest):

    def test_bootstrap(self):
        """
        'ensemble-control bootstrap' will invoke the bootstrap method of all
        configured machine providers in all environments.
        """
        config = {
            "ensemble": "environments",
            "environments": {
                "firstenv": {
                    "type": "dummy", "admin-secret": "homer"},
                "secondenv": {
                    "type": "dummy", "admin-secret": "marge"}}}
        self.write_config(dump(config))
        self.setup_cli_reactor()
        self.setup_exit(0)

        envs = set(("firstenv", "secondenv"))

        def track_bootstrap_call(self):
            envs.remove(self.environment_name)
            return succeed(True)

        provider = self.mocker.patch(MachineProvider)

        provider.bootstrap()
        self.mocker.call(track_bootstrap_call, with_object=True)

        provider.bootstrap()
        self.mocker.call(track_bootstrap_call, with_object=True)
        self.mocker.replay()

        self.capture_stream("stderr")
        main(["bootstrap"])

        lines = filter(None, self.log.getvalue().split("\n"))
        self.assertEqual(
            lines,
            ["Bootstrapping environment 'firstenv' (type: dummy)",
             "Bootstrapping environment 'secondenv' (type: dummy)",
             "'bootstrap' command finished successfully"])
